#include <string>
#include <stdio.h>
#include <map>
#include <Base64.h> // from libSockets
#include <Uid.h>
#include <vector>
#include "rowo.h"
#include "robotworld.h"


void parse_file(const std::string& filename);
const std::string& GetCodestr();
std::map<std::string, int>& GetCodemap();
std::vector<VARIABLE *>& GetVariables();
bool GetFailed();


std::string GetMyUid()
{
	std::string myuid;
	FILE *fil;
	if ((fil = fopen("myuid.txt","rt")) != NULL)
	{
		char slask[1000];
		fgets(slask,1000,fil);
		slask[strlen(slask) - 1] = 0;
		fclose(fil);
		myuid = slask;
	}
	else
	{
		Uid uid;
		myuid = uid.GetUid();
		fil = fopen("myuid.txt","wt");
		fprintf(fil,"%s\n",myuid.c_str());
		fclose(fil);
	}
	return myuid;
}

int main(int argc,char *argv[])
{
	std::string myuid = GetMyUid();

	if (argc < 2)
		printf("Usage: %s <.r file(s)>\n",*argv);
	for (int i = 1; i < argc; i++)
	{
		std::string filename = argv[i];
		unsigned char buf[10000];
		if (filename.substr(filename.size() - 2) != ".r")
		{
			printf("'%s' is not a .r file\n",argv[i]);
		}
		else
		{
			std::string robot_name = filename.substr(0,filename.size() - 2);
			parse_file(argv[i]);

			if (!GetFailed())
			{
			size_t proglen = GetCodestr().size() / 2;
			for (size_t i = 0; i < GetCodestr().size(); i += 2)
			{
				char c1 = GetCodestr()[i];
				char c2 = GetCodestr()[i + 1];
				buf[i / 2] = (c1 - 48 - ((c1 > '9') ? 7:0) - ((c1 > 'F') ? 32:0)) * 16 + 
					     (c2 - 48 - ((c2 > '9') ? 7:0) - ((c2 > 'F') ? 32:0));
			}
			Base64 bb;
			std::string codestr64;
			bb.encode(buf, proglen, codestr64, false);

			// save robot xml
			FILE *fil;
			filename = robot_name + ".xml";
			if ((fil = fopen(filename.c_str(),"wt")) != NULL)
			{
				fprintf(fil,"<?xml version='1.0'?>\n");
				fprintf(fil,"<robot>\n");
				fprintf(fil,"<name value='%s' />\n",robot_name.c_str());
				fprintf(fil,"<owner value='%s' />\n",myuid.c_str());
				fprintf(fil,"<code value=\"%s\" />\n",codestr64.c_str());
				fprintf(fil,"<methods>\n");
				for (std::map<std::string,int>::iterator it = GetCodemap().begin(); it != GetCodemap().end(); it++)
				{
					std::string method = (*it).first;
					int ptr = (*it).second;
					fprintf(fil,"<method name='%s' entry='%d' />\n",method.c_str(),ptr);
				}
				fprintf(fil,"</methods>\n");
				fprintf(fil,"<variables>\n");
				for (std::vector<VARIABLE *>::iterator it = GetVariables().begin(); it != GetVariables().end(); it++)
				{
					VARIABLE *p = *it;
					fprintf(fil,"<variable name='%s' type='%d' index='%d' />\n",
						p -> name.c_str(),p -> type,p -> nr);
				}
				fprintf(fil,"</variables>\n");
				//
				fprintf(fil,"</robot>\n");
				fclose(fil);
			}
			} // if (!failed)
		}
	}
}


